package com.hxstrive.demo_netflix_feign.feign;

import com.alibaba.fastjson.JSONObject;
import com.hxstrive.demo_netflix_feign.entity.User;
import feign.*;
import feign.codec.DecodeException;
import feign.codec.Decoder;
import feign.codec.EncodeException;
import feign.codec.Encoder;
import feign.gson.GsonDecoder;
import feign.gson.GsonEncoder;
import feign.jackson.JacksonDecoder;
import feign.jackson.JacksonEncoder;
import org.springframework.util.StringUtils;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

/**
 * 请求编码/解码器
 * @author hxstrive.com
 * @since 1.0.0  2024/11/1 16:34
 */
public interface EncodeDecodeFeign {

    @RequestLine("POST /simple/encode")
    @Headers("Content-Type: application/json")
    User encode(User user);


    // 创建Feign客户端实例的静态方法
    // Jackson
//    static EncodeDecodeFeign create() {
//        return Feign.builder()
//                .encoder(new JacksonEncoder())
//                .decoder(new JacksonDecoder())
//                .contract(new Contract.Default())
//                .target(EncodeDecodeFeign.class, "http://localhost:8090");
//    }

    // Gson
//    static EncodeDecodeFeign create() {
//        return Feign.builder()
//                .encoder(new GsonEncoder())
//                .decoder(new GsonDecoder())
//                .contract(new Contract.Default())
//                .target(EncodeDecodeFeign.class, "http://localhost:8090");
//    }


    // 自定义编码/解码器
    static EncodeDecodeFeign create() {
        return Feign.builder()
                .encoder(new Encoder() {
                    @Override
                    public void encode(Object o, Type type, RequestTemplate requestTemplate) throws EncodeException {
                        if(Objects.nonNull(o)) {
                            requestTemplate.body(JSONObject.toJSONString(o));
                        } else {
                            requestTemplate.body("");
                        }
                    }
                })
                .decoder(new Decoder(){
                    @Override
                    public Object decode(Response response, Type type) throws IOException, DecodeException, FeignException {
                        try {
                            ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
                            outputStream.write(response.body().asInputStream().readAllBytes());
                            String body = outputStream.toString(StandardCharsets.UTF_8);
                            if (StringUtils.hasText(body)) {
                                return JSONObject.parseObject(body, Class.forName(type.getTypeName()));
                            } else {
                                return null;
                            }
                        } catch (Exception e) {
                            throw new DecodeException(response.status(), "Failed to decode response body: "
                                    + e.getMessage(), response.request(), e);
                        }
                    }
                })
                .contract(new Contract.Default())
                .target(EncodeDecodeFeign.class, "http://localhost:8090");
    }

}
