package com.hxstrive.demo_netflix_feign.feign;

import feign.*;
import feign.codec.Decoder;
import feign.codec.Encoder;

/**
 * Feign 请求拦截器
 * @author hxstrive.com
 * @since 1.0.0  2024/11/1 15:12
 */
public interface InterceptorFeign {

    // Feign 请求接口方法
    @RequestLine("GET /simple/interceptor?msg={msg}")
    String interceptor(@Param("msg") String message);

    // 自定义请求拦截器
    static class ForwardedForInterceptor implements RequestInterceptor {
        @Override
        public void apply(RequestTemplate template) {
            template.header("X-Forwarded-For", "origin.host.com");
        }
    }

    // 创建Feign客户端实例的静态方法
    static InterceptorFeign create() {
        return Feign.builder()
                .encoder(new Encoder.Default())
                .decoder(new Decoder.Default())
                .contract(new Contract.Default())
                // 添加请求拦截器
                .requestInterceptor(new ForwardedForInterceptor())
                .target(InterceptorFeign.class, "http://localhost:8090");
    }

}
