package com.hxstrive.demo_netflix_feign.feign;

import com.alibaba.fastjson.JSONObject;
import com.hxstrive.demo_netflix_feign.entity.User;
import feign.*;
import feign.codec.Decoder;
import feign.codec.Encoder;
import feign.jackson.JacksonDecoder;
import feign.jackson.JacksonEncoder;

import java.util.List;
import java.util.Map;

/**
 * 用户 Feign
 * @author hxstrive.com
 */
@Headers("Custom-Header: SimpleFeign.java")
public interface SimpleFeign {

    // 定义获取用户信息的方法
    @RequestLine("GET /simple/hello")
    public String get();

    @RequestLine(value = "GET /simple/hello2?args={args}", collectionFormat = CollectionFormat.CSV)
    public String get2(@Param("args") List<String> args);

    @RequestLine(value = "GET {path}", decodeSlash = true)
    public String get3(@Param("path") String encodePath);

    // @Param 注解
    @RequestLine("GET /simple/param1?token={token}")
    String param1(@Param("token") String token);

    @RequestLine("GET /simple/param2")
    @Headers("Authorization: Bearer {token}")
    String param2(@Param("token") String token);

    @RequestLine("POST /simple/param3")
    @Body("{body}")
    String param3(@Param("body") String body);

    @RequestLine("POST /simple/param3")
    @Body("{body}")
    String param4(@Param(value = "body", expander = MyExpander.class) String body);

    class MyExpander implements Param.Expander {
        @Override
        public String expand(Object value) {
            return "123" + value + "321";
        }
    }

    @RequestLine("GET /simple/header1")
    @Headers({
            "Authorization: Bearer {token}",
            "Content-Type: text/plain"
    })
    String header1(@Param("token") String token);


    @RequestLine("GET /simple/header1")
    String header2(@HeaderMap Map<String,String> headers);


    @RequestLine("POST /simple/body1")
    @Headers("Content-Type: application/json")
    @Body("{body}")
    String body1(@Param(expander = JSONExpander.class) Map<String,String> body);

    static class JSONExpander implements Param.Expander {
        @Override
        public String expand(Object value) {
            return JSONObject.toJSONString(value);
        }
    }


    @RequestLine("POST /simple/body2")
    @Headers("Content-Type: application/json")
    @Body("{body}")
    User body2(User user);

    @RequestLine("POST /simple/body2")
    @Headers("Content-Type: application/json")
    @Body("{\"id\":100, \"name\":\"Tom\", \"age\":20}")
    User body3();


//    @FeignIgnore
    String ignore();

    @RequestLine("GET /simple/query")
    @Experimental
    User query(@QueryMap User user);


    // 创建Feign客户端实例的静态方法
    static SimpleFeign createJson() {
        return Feign.builder()
                .encoder(new JacksonEncoder())
                .decoder(new JacksonDecoder())
                .contract(new Contract.Default())
                .target(SimpleFeign.class, "http://localhost:8090");
    }

    static SimpleFeign create() {
        return Feign.builder()
                .encoder(new Encoder.Default())
                .decoder(new Decoder.Default())
                .contract(new Contract.Default())
                .target(SimpleFeign.class, "http://localhost:8090");
    }

}
