package com.hxstrive.demo_netflix_feign.feign;

import com.hxstrive.demo_netflix_feign.dto.CommonReturn;
import com.hxstrive.demo_netflix_feign.entity.User;
import feign.*;
import feign.jackson.JacksonDecoder;
import feign.jackson.JacksonEncoder;
import java.util.List;

/**
 * Feign客户端接口
 * @author hxstrive.com
 */
public interface UserFeign {

    @RequestLine("GET /user/getAllUsers")
    CommonReturn<List<User>> getAllUsers();

    @RequestLine("GET /user/getUserById?id={id}")
    CommonReturn<User> getUserById(@Param("id") Long id);

    @RequestLine("POST /user/createUser")
    @Headers("Content-Type: application/json")
    CommonReturn<User> createUser(User user);

    @RequestLine("PUT /user/updateUser?id={id}")
    @Headers("Content-Type: application/json")
    CommonReturn<User> updateUser(@Param("id") Long id, User updatedUser);

    @RequestLine("DELETE /user/deleteUser?id={id}")
    CommonReturn<String> deleteUser(@Param("id") Long id);


    // 创建Feign客户端实例的静态方法
    static UserFeign create() {
        return Feign.builder()
                .encoder(new JacksonEncoder())
                .decoder(new JacksonDecoder())
                .contract(new Contract.Default())
                .target(UserFeign.class, "http://localhost:8090/");
    }

}
