package com.hxstrive.demo_netflix_feign.feign;

import com.hxstrive.demo_netflix_feign.dto.CommonReturn;
import com.hxstrive.demo_netflix_feign.entity.User;
import feign.*;
import feign.jackson.JacksonDecoder;
import feign.jackson.JacksonEncoder;
import java.util.List;

/**
 * UserRestful Feign 客户端
 * @author hxstrive.com
 */
public interface UserRestfulFeign {

    @RequestLine("GET /user/restful/getAllUsers")
    CommonReturn<List<User>> getAllUsers();

    @RequestLine("GET /user/restful/{id}")
    CommonReturn<User> getUserById(@Param("id") Long id);

    @RequestLine("POST /user/restful/createUser")
    @Headers("Content-Type: application/json")
    CommonReturn<User> createUser(User user);

    @RequestLine("PUT /user/restful/{id}")
    @Headers("Content-Type: application/json")
    CommonReturn<User> updateUser(@Param Long id, User updatedUser);

    @RequestLine("DELETE /user/restful/{id}")
    CommonReturn<String> deleteUser(@Param Long id);


    // 创建Feign客户端实例的静态方法
    static UserRestfulFeign create() {
        return Feign.builder()
                .encoder(new JacksonEncoder())
                .decoder(new JacksonDecoder())
                .contract(new Contract.Default())
                .target(UserRestfulFeign.class, "http://localhost:8090/");
    }

}
