package com.hxstrive.service_demo.controller;

import com.hxstrive.service_demo.entity.User;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.util.Map;
import java.util.UUID;

/**
 * 简单控制器
 * @author hxstrive.com
 */
@RestController
@RequestMapping("/simple")
public class SimpleController {

    @Value("${spring.application.name}")
    private String appName;

    @Value("${server.port}")
    private String appPort;

    @GetMapping("/hello")
    public String hello(HttpServletRequest request) {
        return "Hello World";
    }

    @GetMapping("/hello2")
    public String hello2(@RequestParam("args") String[] args) {
        return "Hello World, args=" + String.join("、", args);
    }

    @GetMapping("/info")
    public String info() {
        return "appName=" + appName + " appPort=" + appPort + " uuid=" + UUID.randomUUID().toString();
    }

    @GetMapping("/get")
    public String get(@RequestParam("id") Long id) {
        return "appName=" + appName + " appPort=" + appPort + " id=" + id;
    }

    @GetMapping("/param1")
    public String param1(@RequestParam("token") String token) {
        return "token=" + token;
    }

    @GetMapping("/param2")
    public String param2(@RequestHeader("Authorization") String token) {
        return "Authorization=" + token;
    }

    @PostMapping("/param3")
    public String param3(@RequestBody String body) {
        return "body=" + body;
    }

    @GetMapping("/header1")
    public String header1(@RequestHeader("Custom-Header") String customHeader,
                          @RequestHeader("Authorization") String token,
                          @RequestHeader("Content-Type") String contentType) {
        return "Custom-Header=" + customHeader +
                "<br/>Authorization=" + token +
                "<br/>Content-Type=" + contentType;
    }

    @PostMapping("/body1")
    public Map<String,String> body1(@RequestBody Map<String,String> body) {
        body.put("service_name", "ServiceDemo");
        return body;
    }

    @PostMapping("/body2")
    public User body2(@RequestBody User user) {
        return user;
    }

    @GetMapping("/query")
    public User query(@RequestParam("id") Long id, @RequestParam("name") String name, @RequestParam("age") Integer age) {
        return User.builder().id(id).name(name).age(age).build();
    }

    @GetMapping("/interceptor")
    public String interceptor(@RequestHeader("X-Forwarded-For") String xForwardedFor, @RequestParam("msg") String msg) {
        return "xForwardedFor=" + xForwardedFor + "<br/>msg=" + msg;
    }

    @PostMapping("/encode")
    public User encode(@RequestBody User user) {
        System.out.println(user);
        return user;
    }

}
